// Use this to generate load-wasm.js; see Makefile.

let SYNC = '(i32.const 48)';  // where workers sleep and main writes
let ACC  = '(i32.const 52)';  // where workers accumulate values read
let DONE = 86;		      // flag value

let text =
`(module
  (memory (import "" "memory") 1 1 shared)

  (func (export "main") (param $val i32) (result i32)
   (i32.atomic.store ${SYNC} (get_local $val))
   (wake ${SYNC} (i32.const -1)))

  (func (export "worker")
   (local $prev i32)
   (loop $out $in
    (drop (i32.wait ${SYNC} (get_local $prev) (i64.const -1)))
    (set_local $prev (i32.atomic.load ${SYNC}))
    (drop (i32.atomic.rmw.add ${ACC} (get_local $prev)))
    (br_if $in (i32.ne (get_local $prev) (i32.const ${DONE}))))))`;

let bin = new Uint8Array(wasmTextToBinary(text));
let s = "var WasmBytes = new Uint8Array([";
s += bin[0];
for ( let i=1; i < bin.length; i++ )
    s += "," + bin[i];
s += "]);";

print(s);

