// Simple test case with shared memory:
//
// - Each worker will create a wasm instance from the received module and
//   memory, and enter a wasm loop that sleeps on a location, reads that
//   location's new value after waking and adds it to a shared counter, and
//   repeats this until the value read is 86, after which they exit.
//
// - The main loop runs a wasm function on a timer that sets up the value and
//   signals the workers, before exiting to JS, which sleeps, and does it again.
//   The main wasm function returns the number of waiters woken, which we check.

let mod = new WebAssembly.Module(WasmBytes);
let mem = new WebAssembly.Memory({initial: 1, maximum: 1, shared: true});
let ins = new WebAssembly.Instance(mod, {"": {memory: mem}});
let main = ins.exports.main;
let buf = new Int32Array(mem.buffer);

let NUM = 10;			// Workers
let ACC = 52 / 4;		// Accumulator location
let DONE = 86;			// Termination value
let STEP = 250;			// Timeout

writeln("Fine so far");

for ( let i=1; i <= NUM; i++ ) {
    let w = new Worker("test-mem-worker.js");
    w.onmessage = evHandler;
    w.postMessage([i, mod, mem]);
}

setTimeout(nextItem, STEP);

let data = [1, 1, 2, 3, 5, 8, 13, 21, 34, DONE];
let dataptr = 0;

function nextItem() {
    let expected = 0;
    for ( let i=0; i < dataptr; i++ )
	expected += data[i];
    expected *= NUM;
    let val = buf[ACC];
    if (val != expected)
	writeln("ERROR! Expected: " + expected);
    writeln("Sum: " + val);
    let v = data[dataptr++];
    let c = main(v);
    if (c != NUM)
	writeln("Bad wakeup count: " + c);
    if (v != DONE)
	setTimeout(nextItem, STEP);
    else
	writeln("Terminating");
}

function evHandler(ev) {
    message(String(ev.data));
}

function writeln(s) {
    message("Main: " + s);
}
